/* 
 * Blinky - a GTK network usage monitor
 *
 * Copyright (C) 2002, Benjamin Miller <bmiller@users.sf.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


/* usage-kstat.c - get device usage using the Solaris kstat interface */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <kstat.h>
#include "blinky.h"


/* Solaris version of get_usage_all -- uses kstat functions to get tx and
   rx counts for all the IFAs */
void
get_usage_all (void)
{
    static kstat_ctl_t *kc = NULL;
    kstat_named_t *kn;
    void *data;
    GSList *l;
    unsigned long tx, rx;

    if (kc == NULL) /* initialize the kstat chain */
	kc = kstat_open();

    l = ifs;
    while (l != NULL) {
	IFA *f = l->data;
	tx = f->tx_cur;
	rx = f->rx_cur;

	if (f->k == NULL) { /* look up the interface by name */
	    f->k = kstat_lookup (kc, NULL, -1, f->name);
	    if (f->k == NULL) {
		/* If it's still NULL, the interface is down or doesn't 
		   exist */
		l = l->next;
		continue;
	    }
	}

	if (kstat_read (kc, f->k, NULL) == -1 ||
	    f->k->ks_type != KSTAT_TYPE_NAMED) {
	    /* unable to read - skip this IFA */
	    l = l->next;
	    continue;
	}

	/* Look up tx count */
	if ((data = kstat_data_lookup (f->k, "obytes"))) {
	    kn = (kstat_named_t *)data;
	    switch (kn->data_type) {
	    case KSTAT_DATA_UINT32:
		tx = kn->value.ui32;
		break;
	    default:
		/* Check other types? */
		break;
	    }
	}

	/* Look up rx count */
	if ((data = kstat_data_lookup (f->k, "rbytes"))) {
	    kn = (kstat_named_t *)data;
	    switch (kn->data_type) {
	    case KSTAT_DATA_UINT32:
		rx = kn->value.ui32;
		break;
	    default:
		/* Check other types? */
		break;
	    }
	}

	/* we found the interface by this point, and it is up */
	f->found = TRUE;
	f->up = TRUE;

	/* value of -1 means reset to the newly read value */
	if (f->tx_cur == -1)
	    f->tx_cur = tx;
	if (f->rx_cur == -1)
	    f->rx_cur = rx;

	/* is the interface transmitting? */
	if (tx > f->tx_cur)
	    f->txing = TRUE;
	else
	    f->txing = FALSE;
	/* is it receiving? */
	if (rx > f->rx_cur)
	    f->rxing = TRUE;
	else
	    f->rxing = FALSE;

	/* set to newly read values */
	f->tx_cur = tx;
	f->rx_cur = rx;

	l = l->next;
    }

    return;
}


/*
 * Local variables:
 * c-basic-offset: 4
 * End:
 */
