/* 
 * Blinky - a GTK network usage monitor
 *
 * Copyright (C) 2002, Benjamin Miller <bmiller@users.sf.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include "getopt.h"
#include "blinky.h"


static char if_name[] = "ppp0";


/* Fills in the status of all IFAs in the list ifs */
gboolean
update_all (gpointer ptr)
{
    GSList *l;
    char tx_str[16], rx_str[16];

    /* first mark all interfaces as not found, then get_usage_all will mark
       them if it finds them */
    l = ifs;
    while (l != NULL) {
	IFA *f = l->data;
	f->found = FALSE;
	l = l->next;
    }

    /* get usage of all IFAs -- OS specific */
    get_usage_all ();

    /* if any interfaces are left unmarked, deal with them */
    l = ifs;
    while (l != NULL) {
	IFA *f = l->data;
	if (f->found == FALSE) {
	    if (f->up) {
		f->up = FALSE;
		snprintf (f->tip, sizeof(f->tip), 
			  "%s: disconnected", f->name);
		gtk_tooltips_set_tip (tips, f->eventbox, 
				      f->tip, "");
	    }
	    f->tx_cur = f->rx_cur = -1;
	    f->txing = f->rxing = f->up = FALSE;
	}
	l = l->next;
    }

    /* redraw all interfaces, reset tooltips if necessary */
    l = ifs;
    while (l != NULL) {
	IFA *f = l->data;
	ifa_draw (f->eventbox, NULL, f);
	l = l->next;

	if (!f->up)
	    continue;

	if (f->tip_timer == 0) {

	    // Added by Kevin :)
	    format_bytes(f->rx_cur, rx_str, sizeof(rx_str));
	    format_bytes(f->tx_cur, tx_str, sizeof(tx_str));

	    snprintf (f->tip, sizeof(f->tip), 
		      "%s: %s received, %s sent",
		      f->name, rx_str, tx_str);
	    // end added by Kevin

	    gtk_tooltips_set_tip (tips, f->eventbox, f->tip, "");
	    f->tip_timer = 20;
	} else
	    f->tip_timer--;
    }
    

    return TRUE;
}


void
get_setup (int argc, char **argv)
{
    char opt;
    IFA *ifa;
    struct option long_options[] = {
	{"help", no_argument, NULL, 'h'},
	{"update", required_argument, NULL, 'u'},
	{"ac", required_argument, NULL, 'A'},
	{"ic", required_argument, NULL, 'I'},
	{"dc", required_argument, NULL, 'D'},
	{0, 0, 0, 0}
    };

    /* initialize global options */
    options = g_malloc (sizeof (blinky_opts));

    /* default update interval is 200 milliseconds */
    options->interval = 200;
    options->ac = options->ic = options->dc = NULL;
    options->agc = options->igc = options->dgc = NULL;

    opterr = 1;
    while ((opt = getopt_long_only(argc, argv, "hu:", 
				   long_options, NULL)) != -1) {
	switch (opt) {
	case 'h':
	    /* show help */
	    printf ("Usage: %s [OPTION]... [IFACE]\n\n"
		    "  -h, --help\t\tdisplay this help and exit\n"
		    "  -u, --update=NUM\tupdate every NUM milliseconds\n"
		    "  -ac, -ic, -dc COLOR\tset colors:\n"
		    "\t\t\t  -ac: interface active, -ic: interface idle,\n"
		    "\t\t\t  -dc: interface down\n"
		    "\nIFACE is a network interface (e.g. eth0, ppp0)\n\n",
		    argv[0]);
	    exit(0);
	    break;
	case 'u':
	    /* 'u' for "update interval" */
	    options->interval = atoi (optarg);
	    break;
	case 'A':
	    /* active color */
	    options->ac = g_malloc (sizeof(GdkColor));
	    if (!gdk_color_parse (optarg, options->ac)) {
		fprintf (stderr, "Warning: Color name "
			 "\"%s\" is not defined\n", optarg);
		g_free (options->ac);
		options->ac = NULL;
	    }
	    break;
	case 'I':
	    /* inactive color */
	    options->ic = g_malloc (sizeof(GdkColor));
	    if (!gdk_color_parse (optarg, options->ic)) {
		fprintf (stderr, "Warning: Color name "
			 "\"%s\" is not defined\n", optarg);
		g_free (options->ic);
		options->ic = NULL;
	    }
	    break;
	case 'D':
	    /* inactive color */
	    options->dc = g_malloc (sizeof(GdkColor));
	    if (!gdk_color_parse (optarg, options->dc)) {
		fprintf (stderr, "Warning: Color name "
			 "\"%s\" is not defined\n", optarg);
		g_free (options->dc);
		options->dc = NULL;
	    }
	    break;
	default:
	    /* print help.. */
	    fprintf (stderr, 
		     "Try `%s --help' for more information.\n",
		     argv[0]);
				 
	    exit(1);
	    break;
	}
    }

    /* if no color options were given, use defaults */
    if (!options->ac) {
	options->ac = g_malloc (sizeof(GdkColor));
	options->ac->red = options->ac->blue = 0;
	options->ac->green = 0xFFFF;
    }
    if (!options->ic) {
	options->ic = g_malloc (sizeof(GdkColor));
	options->ic->red = options->ac->blue = 0;
	options->ic->green = 0x8000;
    }
    if (!options->dc) {
	options->dc = g_malloc (sizeof(GdkColor));
	options->dc->green = options->ac->blue = 0;
	options->dc->red = 0x8000;
    }

    /* remaining arguments are interface names.. */
    if (optind < argc) {
	strncpy (if_name, argv[optind], sizeof(if_name));
	optind++;
    }
    /* always do at least one interface - if none given, use default */
    ifa = ifa_new (if_name);
    ifs = g_slist_append (ifs, ifa);

    /* now see if there are any more */
    while (optind < argc) {
	ifa = ifa_new (argv[optind]);
	ifs = g_slist_append (ifs, ifa);
	optind++;
    }

    return;
}


GdkGC *
color_to_gc (GtkWidget *widget, GdkColor *color)
{
    static GdkColormap *cmap = NULL;
    static GdkVisual *vis = NULL;
    GdkGC *gc;

    if (cmap == NULL)
	cmap = gtk_widget_get_colormap (widget);
    if (vis == NULL)
	vis = gdk_colormap_get_visual (cmap);
    gdk_color_alloc (cmap, color);
    gc = gdk_gc_new (widget->window);
    gdk_gc_set_foreground (gc, color);

    return gc;
}

/* This function takes a unsigned long long number of bytes, and writes
 * a nicly formatted version of the size in Kb, MB, GB, etc... into a provided
 * buffer.  To insure no buffer overflows occur one must also pass in the size
 * of the buffer provided, which at maximum should only need to be 11 including
 * the terminating NULL */
void format_bytes (unsigned long long bytes, char *buffer, int size) 
{
    if(bytes > 1024) { // KB range 
	if(bytes > 1048576) { // MB range 
	    if(bytes > 1073741824) { // GB range
		if(bytes > 1099511627776) { // TB range
		    if(bytes > 1125899906842624) { // PB range
			if(bytes > 1152921504606846976) { // EB range
			    if(bytes > 1180591620717411303424) { // ZB range
				/* Seems these numbers are too big for standard llu

			        if(bytes > 12089258196146291747066176) { // YB range
				    snprintf (buffer, size, 
				    "%.2f YB",
				    (float)bytes/12089258196146291747066176);
				} else  // ZB range

				*/
				snprintf (buffer, size, 
					  "%.2f ZB",
					  (float)bytes/1180591620717411303424);
				
			    } else  // EB range
				snprintf (buffer, size, 
					  "%.2f EB",
					  (float)bytes/1152921504606846976);
			} else // PB range
			    snprintf (buffer, size, 
				      "%.2f PB",
				      (float)bytes/1125899906842624);
		    } else // TB range
			snprintf (buffer, size, 
				  "%.2f TB",
				  (float)bytes/1099511627776);
		} else // GB range
		    snprintf (buffer, size, 
			      "%.2f GB",
			      (float)bytes/1073741824);
	    } else //MB range
		snprintf (buffer, size, 
			  "%.2f MB",
			  (float)bytes/1048576);
	} else //KB range
	    snprintf (buffer, size, 
		      "%.2f KB",
		      (float)bytes/1024);
    } else // byte range
	snprintf (buffer, size, 
		  "%llu bytes",
		  bytes);
    return;
}


/*
 * Local variables:
 * c-basic-offset: 4
 * End:
 */
